/*
 *
 *			   IPSEC for Linux
 *		         Preliminary Release
 * 
 *	 Copyright (C) 1996, 1997, John Ioannidis <ji@hol.gr>
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
 * USA
 *
 */

/*
 * $Id: ipsec_espdesmd596.c,v 0.5 1997/06/03 04:24:48 ji Rel $
 *
 * $Log: ipsec_espdesmd596.c,v $
 * Revision 0.5  1997/06/03 04:24:48  ji
 * New file.
 *
 *
 */

#include <linux/config.h>
#include <asm/segment.h>
#include <asm/system.h>
#include <linux/types.h>
#include <linux/kernel.h>
#include <linux/sched.h>
#include <linux/string.h>
#include <linux/errno.h>
#include <linux/config.h>

#include <linux/socket.h>
#include <linux/sockios.h>
#include <linux/in.h>
#include <linux/inet.h>
#include <linux/netdevice.h>
#include <linux/etherdevice.h>
#include <linux/icmp.h>
#include <linux/udp.h>
#include <net/ip.h>
#include <net/protocol.h>
#include <net/route.h>
#include <net/tcp.h>
#include <net/udp.h>
#include <net/sock.h>
#include <net/icmp.h>

#include <net/checksum.h>

#include <linux/fs.h>
#include <linux/mm.h>
#include <linux/miscdevice.h>

#include <linux/skbuff.h>
#include <linux/proc_fs.h>
#include <linux/stat.h>

#include <net/netlink.h>
#include <unistd.h>
#include "radij.h"
#include "ipsec_encap.h"
#include "ipsec_radij.h"
#include "ipsec_netlink.h"
#include "ipsec_xform.h"
#ifdef CONFIG_IPSEC_AH
#include "ipsec_ah.h"
#endif
#ifdef CONFIG_IPSEC_ESP
#include "ipsec_esp.h"
#endif

extern void des_cbc_encrypt(caddr_t, caddr_t, int, caddr_t, caddr_t, int);
extern void des_set_key(caddr_t, caddr_t);

#ifdef DEBUG_IPSEC_ESP
static void
print_ip_pkt(struct iphdr *ip)
{
	int i, len;
	unsigned char *d = (unsigned char *)ip;
	len = ntohs(ip->tot_len);
	
	printk("packet length: %d", len);
	for (i = 0; i < len; i++)
		printk(" %02x", d[i]);
	printk("\n");
}

	
#ifdef NOLONGER_DEBUG_IPSEC_ESP
static void
print_ip(struct iphdr *ip)
{
	unsigned char *ipaddr;

	printk("IP packet:\n");
	printk("--- header len = %d\n", ip->ihl*4);
	printk("--- ip version: %d\n", ip->version);
	printk("--- ip protocol: %d\n", ip->protocol);
	ipaddr=(unsigned char *)&ip->saddr;
	printk("--- source address: %u.%u.%u.%u\n", 
			*ipaddr, *(ipaddr+1), *(ipaddr+2), *(ipaddr+3));
	ipaddr=(unsigned char *)&ip->daddr;
	printk("--- destination address: %u.%u.%u.%u\n", 
			*ipaddr, *(ipaddr+1), *(ipaddr+2), *(ipaddr+3));
	printk("--- total packet len: %d\n", ntohs(ip->tot_len));
}
#endif
#endif

/*
 * Check-replay-window routine, adapted from the original 
 * by J. Hughes, from draft-ietf-ipsec-esp-des-md5-03.txt
 *
 *  This is a routine that implements a 64 packet window. This is intend-
 *  ed on being an implementation sample.
 */

static int
esp_checkreplaywindow(struct espdesmd5_xdata *xd, unsigned long seq)
{
	unsigned long diff;
	
	/* If manual keying, don't really check replay */
	if (xd->edmx_flags & EME_MANUAL)
		return 1;
	if (seq == 0) 
		return 0;		/* first == 0 or wrapped */

	if (seq > xd->edmx_lastseq)	/* new larger sequence number */
	{
		diff = seq - xd->edmx_lastseq;
		if (diff < xd->edmx_ooowin) /* In win, set bit for this pkt */
			xd->edmx_bitmap = (xd->edmx_bitmap << diff) | 1;
		else
			xd->edmx_bitmap = 1; /* This packet has way larger */

		xd->edmx_lastseq = seq;
		return 1;		/* larger is good */
	}
	diff = xd->edmx_lastseq - seq;
	if (diff >= xd->edmx_ooowin)	/* too old or wrapped */
		return 0;
	
	if (xd->edmx_bitmap & (1 << diff)) /* this packet already seen */
		return 0;
	xd->edmx_bitmap |= (1 << diff);	/* mark as seen */
	return 1;			/* out of order but good */
}

struct sk_buff *
espdesmd596_input(struct sk_buff *skb, struct tdb *tdbp)
{
	struct iphdr *ipp;
	struct espdesmd5_xdata *xd;
	u_long *espp, seq;
	int iphlen, esphlen, proto, len, pad, ilen;

	MD5_CTX tctx;
	u_char mdchk[16];

	unsigned char *dat, *idat;
	u_long iv[2], iv2[2];
	
	len = skb->len; dat = skb->data;

	ipp = (struct iphdr *)dat;
	iphlen = ipp->ihl << 2;
	espp = (u_long *)(dat + iphlen);
	
	xd = (struct espdesmd5_xdata *)tdbp->tdb_xdata;

	idat = dat + iphlen;
	ilen = len - iphlen - 12;

	if (ilen % 8)
	{
		printk("ipsec_espdes: got packet
 with ilen = %d \n", ilen);
		return NULL;
	}

	seq = ntohl(*((unsigned long *)(&idat[4]))); 

	if (!esp_checkreplaywindow(xd, seq))
	{
#ifdef DEBUG_IPSEC_ESP
		if (debug_esp & DB_ES_REPLAY)
			printk("espdesmd596_input: duplicate frame\n");
#endif
		return NULL;
	}

	/*
	 * verify authenticator
	 */

	tctx = xd->edmx_ictx;
	MD5Update(&tctx, (caddr_t)espp, ilen); /* HMAC first four octets */
	MD5Final(mdchk, &tctx);
	tctx = xd->edmx_octx;
	MD5Update(&tctx, mdchk, sizeof mdchk);
	MD5Final(mdchk, &tctx);

	if (memcmp(mdchk, &(dat[len - 12]), 12))
	{
#ifdef DEBUG_IPSEC_ESP
		if (debug_esp & DB_ES_INAU)
			printk("espdesmd596_input: bad auth\n");
#endif
		return NULL;
	}

	if (xd->edmx_flags & EME_CONSTIV)
	{
		iv[0] = xd->edmx_iv[0];
		iv[1] = xd->edmx_iv[1];
		esphlen = 8;
	}
	else
	{
		iv[0] = espp[2];
		iv[1] = espp[3];
		esphlen = 16;
	}
	iv2[0] = iv[0]; iv2[1] = iv[1];

	idat += esphlen;
	ilen -= esphlen;

	des_cbc_encrypt(idat, idat, ilen, (caddr_t)(xd->edmx_eks), (caddr_t)iv, 0);

	proto = idat[ilen - 1];
	pad = idat[ilen - 2] + 2 + 12;
	
#ifdef DEBUG_IPSEC_ESP
	if (debug_esp & DB_ES_IPAD)
	  printk("espdesmd596_input: proto = %d, padding = %d\n", proto, pad);
#endif

	/*
	 *	Discard the original IP header
	 */
	 
	ipp->tot_len = htons(ntohs(ipp->tot_len) - esphlen - pad);
	ipp->protocol = proto;

	memmove((void *)(skb->data + esphlen), (void *)skb->data, iphlen);

	skb_pull(skb, esphlen);
	skb_trim(skb, len - esphlen - pad);

	/*
	 *	Adjust pointers
	 */
	 
	
	len = skb->len; dat = skb->data;
	

	skb->h.iph=(struct iphdr *)skb->data;
	skb->ip_hdr=(struct iphdr *)skb->data;

	memset(skb->proto_priv, 0, sizeof(struct options));
	
	ipp = (struct iphdr *)dat;
	ipp->check = 0;
	ipp->check = ip_fast_csum((unsigned char *)dat, iphlen >> 2);

	return skb;
}

int
espdesmd596_attach(void)
{
	printk("espdesmd596_attach: called.\n");
	return 0;
}

/*
 * espdes_init() is called when an SPI is being set up. It interprets the
 * encap_msghdr present in m, and sets up the transformation data.
 */

int
espdesmd596_init(struct tdb *tdbp, struct xformsw *xsp, struct encap_msghdr *em)
{
	struct espblkrply_edata *ed;
	struct espdesmd5_xdata *xd;
        /* XXX This should be dynamically allocated or something */
	__u32 mdoutput[16];		/* temp space for MD5 digests */

	tdbp->tdb_xform = xsp;

	if ((em->em_msglen - EMT_SETSPI_FLEN) > sizeof (struct espblkrply_edata))
		return EINVAL;

	tdbp->tdb_xdata = (caddr_t)kmalloc(sizeof (struct espdesmd5_xdata), GFP_ATOMIC); /* XXX memory leak here if updating old TDB? */

	if (tdbp->tdb_xdata == NULL)
	  return ENOBUFS;
	memset(tdbp->tdb_xdata, 0, sizeof (struct espdesmd5_xdata));

	xd = (struct espdesmd5_xdata *)tdbp->tdb_xdata;
	ed = (struct espblkrply_edata *)em->em_dat;

	xd->edmx_flags = ed->eme_flags;
	xd->edmx_ooowin = ed->eme_ooowin;
	
        des_set_key((caddr_t)ed->eme_key, (caddr_t)(xd->edmx_eks));

	xd->edmx_iv[0] = ed->eme_ivl[0];
	xd->edmx_iv[1] = ed->eme_ivl[1];

        MD5Init(&xd->edmx_ictx);

        if (ed->eme_klen - 8 > 64)
        {
		memset((caddr_t)mdoutput, 0, sizeof mdoutput);
                MD5Update(&xd->edmx_ictx, (caddr_t)ed->eme_key + 8,
			  ed->eme_klen - 8);
                MD5Final((caddr_t)mdoutput, &xd->edmx_ictx);
                MD5Init(&xd->edmx_ictx);
        }
        else
          memmove((caddr_t)mdoutput, (caddr_t)ed->eme_key + 8, ed->eme_klen - 8);

	printk("%d bytes of auth key\n", ed->eme_klen - 8);

	mdoutput[0] ^= 0x36363636;	mdoutput[1] ^= 0x36363636;
	mdoutput[2] ^= 0x36363636;	mdoutput[3] ^= 0x36363636;
	mdoutput[4] = 0x36363636;	mdoutput[5] = 0x36363636;
	mdoutput[6] = 0x36363636;	mdoutput[7] = 0x36363636;
	mdoutput[8] = 0x36363636;	mdoutput[9] = 0x36363636;
	mdoutput[10] = 0x36363636;	mdoutput[11] = 0x36363636;
	mdoutput[12] = 0x36363636;	mdoutput[13] = 0x36363636;
	mdoutput[14] = 0x36363636;	mdoutput[15] = 0x36363636;
	
	MD5Update(&xd->edmx_ictx, (unsigned char *)mdoutput, sizeof mdoutput);
		
	mdoutput[0] ^= 0x6a6a6a6a;	mdoutput[1] ^= 0x6a6a6a6a;
	mdoutput[2] ^= 0x6a6a6a6a;	mdoutput[3] ^= 0x6a6a6a6a;
	mdoutput[4] ^= 0x6a6a6a6a;	mdoutput[5] ^= 0x6a6a6a6a;
	mdoutput[6] ^= 0x6a6a6a6a;	mdoutput[7] ^= 0x6a6a6a6a;
	mdoutput[8] ^= 0x6a6a6a6a;	mdoutput[9] ^= 0x6a6a6a6a;
	mdoutput[10] ^= 0x6a6a6a6a;	mdoutput[11] ^= 0x6a6a6a6a;
	mdoutput[12] ^= 0x6a6a6a6a;	mdoutput[13] ^= 0x6a6a6a6a;
	mdoutput[14] ^= 0x6a6a6a6a;	mdoutput[15] ^= 0x6a6a6a6a;
	
	MD5Init(&xd->edmx_octx);
	MD5Update(&xd->edmx_octx, (unsigned char *)mdoutput, sizeof mdoutput);

	xd->edmx_lastseq = 1;

	return 0;
}



int
espdesmd596_zeroize(void)
{
	return 0;
}

int
espdesmd596_print(void *xdat, char *buf)
{
	int size;
	int i, ds;
	union 
	{
		u_char c[8];
		u_long l[2];
	}cl;

	struct espdesmd5_xdata *xd = (struct espdesmd5_xdata *)xdat;
	size = sprintf(buf, "iv =");
	buf += size;

	cl.l[0] = xd->edmx_iv[0];
	cl.l[1] = xd->edmx_iv[1];
	
	for (i = 0; i < 8; i++)
	{
		ds = sprintf(buf, " %02x", cl.c[i]);
		buf += ds;
		size += ds;
	}

	ds = sprintf(buf, " seq = 0x%08x, bit = %08x, win = %d", xd->edmx_lastseq, xd->edmx_bitmap, xd->edmx_ooowin);
	buf += ds;
	size += ds;
	

	ds = sprintf(buf, " flags = %x <", xd->edmx_flags);
	buf += ds;
	size += ds;

	if (xd->edmx_flags & EME_CONSTIV)
	{
		ds = sprintf(buf, "CONSTIV,");
		buf += ds;
		size += ds;
	}

        if (xd->edmx_flags & EME_MANUAL)
        {
                ds = sprintf(buf, "MANUAL,");
                buf += ds;
                size += ds;
        }

	/* 
	 * (other flag printing goes here)
	 */

	if (xd->edmx_flags & EME_INITIATOR)
		ds = sprintf(buf, "INITIATOR>");
	else
		ds = sprintf(buf, "RESPONDER>");
	buf += ds;
	size += ds;

#ifdef ESPPRINTKEYS
	ds = sprintf(buf, ", ks = ");
	buf += ds;
	size += ds;
	
	for (i = 8; i < 136; i++)
	{
		ds = sprintf(buf, " %02x", xd->edmx_iv[i]);
		buf += ds;
		size += ds;
	}
#endif
	return size;
}

int
espdesmd596_room(struct tdb *tp, int iphlen, int tlen, int *hr, int *tr)
{
	struct espdesmd5_xdata *xd = (struct espdesmd5_xdata *)(tp->tdb_xdata) ;
	
	*hr += 4 + 4 			/* SPI + count */ 
		+ ((xd->edmx_flags & EME_CONSTIV) ? 0 : 8); /* IV */
		
	/* the +4 accounts for the length of the counter */
	*tr += ((8 - ((tlen + 2) % 8)) % 8) + 2 /* padding */
		+ 12;			/* 96-bit truncated MD5 HMAC */
	
	return 0;
}

int
espdesmd596_output(struct sk_buff *skb, struct tdb *tdbp)
{
	struct iphdr *ipp;
	struct espdesmd5_xdata *xd;
	__u32 *espp;
	int iphlen, esphlen, hrm, trm, len, ilen;
	unsigned char *dat, *idat;
	MD5_CTX tctx;
	__u32 thash[4];
	__u32 iv[2];
	
	ipp = (struct iphdr *)(skb->data);
#ifdef DEBUG_IPSEC_ESP
	if (debug_esp & DB_ES_OINFO)
	{
		printk("espdesmd596_output: old header (headroom = %d, tailroom = %d\n", skb_headroom(skb), skb_tailroom(skb));
		print_ip_pkt(ipp);
	}

#endif	
	iphlen = ipp->ihl << 2;
	if (iphlen != sizeof (struct iphdr))
	{
		printk("espdesmd596_output: cannot process options yet\n"); /* XXX */
		return -1;
	}

	xd = (struct espdesmd5_xdata *)(tdbp->tdb_xdata);

	/* see espdesmd596_room() for next two lines */

	hrm = 8 + ((xd->edmx_flags & EME_CONSTIV) ? 0 : 8);
	trm = ((8 - ((skb->len - iphlen + 2) % 8)) % 8) + 2 + 12;

	dat = skb_push(skb, hrm);
	ilen = skb->len;
	len = ilen + trm;
	
#ifdef DEBUG_IPSEC_ESP
	if (debug_esp & DB_ES_OINFO2)
		printk("before skb_trim: %d,%d (need: %d,%d)", skb_headroom(skb), skb_tailroom(skb), hrm, trm);
#endif
	skb_put(skb, trm);
#ifdef DEBUG_IPSEC_ESP
	if (debug_esp & DB_ES_OINFO2)
		printk(" after skb_trim: %d,%d\n", skb_headroom(skb), skb_tailroom(skb));
#endif	
	memmove((void *)dat, (void *)(dat + hrm), iphlen);
	espp = (__u32 *)(dat + iphlen);
	ipp = (struct iphdr *)dat;

	if (ntohs(ipp->frag_off) & IP_OFFSET)
	{
		printk("espdesmd596_output: attempted to encrypt fragment!\n");
		return -1;
	}

	espp[0] = tdbp->tdb_spi;
	espp[1] = htonl(xd->edmx_lastseq++);

	if (xd->edmx_flags & EME_CONSTIV)
	{
		esphlen = 8;		/* what's not encrypted */
		iv[0] = xd->edmx_iv[0];
		iv[1] = xd->edmx_iv[1];
	}
	else
	{
		esphlen = 16;		/* what's not encrypted */
		iv[0] = espp[2] = xd->edmx_iv[0];
		iv[1] = espp[3] = xd->edmx_iv[1];
		if ((xd->edmx_iv[1]++ == 0) && (xd->edmx_iv[0]++ == 0))
			xd->edmx_iv[1] = 1; /* wrapped around!!! */
	}

	dat[len - 12 - 1] = ipp->protocol;
	dat[len - 12 - 2] = trm - 2 - 12;

	idat = dat + iphlen + esphlen;
	ilen = len - iphlen - esphlen;

#ifdef DEBUG_IPSEC_ESP
	if (debug_esp & DB_ES_OINFO)
		printk("encrypting %d bytes\n", ilen);
#endif	
	des_cbc_encrypt(idat, idat, ilen, (caddr_t)(xd->edmx_eks), (caddr_t)iv, 1);


	tctx = xd->edmx_ictx;


	MD5Update(&tctx, (caddr_t)espp, len - iphlen - 12);
	MD5Final((unsigned char *)thash, &tctx);

	tctx = xd->edmx_octx;
	MD5Update(&tctx, (unsigned char *)thash, sizeof thash);
	MD5Final((unsigned char *)thash, &tctx);
	memmove(&(dat[len - 12]), thash, 12);

	ipp->protocol		=	50;
	ipp->tot_len		=	htons(len);

	skb->ip_hdr = skb->h.iph = (struct iphdr *) skb->data;
	ipp->check = 0;
	ipp->check = ip_fast_csum((unsigned char *)ipp, ipp->ihl);

#ifdef DEBUG_IPSEC_ESP
	if (debug_esp & DB_ES_OH)
	{
		printk("espdesmd596_output: new header (headroom = %d, tailroom = %d\n", skb_headroom(skb), skb_tailroom(skb));
		print_ip_pkt(ipp);
	}
#endif


	return 0;
}

